package com.miyzh.service;

import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.miyzh.dao.MedicineStorageDao;
import com.miyzh.entity.*;
import com.miyzh.entity.dto.MedicineStorageDto;
import com.miyzh.entity.dto.MedicineStoreDto;
import com.miyzh.utils.RestTemplateUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * 入库
 */

@Slf4j
@Service
public class MedicineStorageService {

    @Autowired
    private MedicineStorageDao medicineStorageDao;
    @Value ("${postApi.sorageUrl}")
    private String url;
    @Value("${postApi.pageSize}")
    private int pageSize;
    @Value("${update.startDate}")
    public String startDate;
    @Value("${update.endDate}")
    public String endDate;
    @Value("${update.dateType}")
    public String dateType;
    @Value("${update.count}")
    public Integer count;

    private class StorageType {
        private static final String PUR = "1"; //采购入库
        private static final String OTHER = "2"; //其他
        private static final String INBOUND = "3"; //销售退货
        private static final String CHI_MED_INBOUND = "4"; // 退溢
    }
    private static final String MED_PUR = "1"; //西药采购入库
    private static final String CHI_MED_PUR = "2"; //中药采购入库
    private static final String MED_MED_INBOUND = "3"; //西药销售退回入库
    private static final String CHI_MED_INBOUND = "4"; //中药销售退回
    private static final String MED_INV = "5"; //西药盘盈入库
    private static final String CHI_MED_INV = "6"; //中药盘盈入库

    @Async
    public void uploadMedicine(List<Clinic> clinicList, String flag) throws InterruptedException, JsonProcessingException {
        long startTime = System.currentTimeMillis();
        PageInfo<MedicineStorageTemp> page = new PageInfo<>();
        for(Clinic clinic : clinicList) {
            switch (flag) {
                case MED_PUR:
                    page = PageHelper.startPage(1, pageSize).doSelectPageInfo(() -> {
                        medicineStorageDao.listMedicinePurchase(clinic.getClinicId(), startDate, endDate, dateType, count);
                    });
                    System.out.println(clinic.getStoreCode() + "======查询西药采购入库列表======total=" + page.getTotal());
                    break;
                case CHI_MED_PUR:
                    page = PageHelper.startPage(1, pageSize).doSelectPageInfo(() -> {
                        medicineStorageDao.listChinaMedicinePurchase(clinic.getClinicId(), startDate, endDate, dateType, count);
                    });
                    System.out.println(clinic.getStoreCode() + "======查询中药采购入库列表======total=" + page.getTotal());
                    break;
                case MED_MED_INBOUND:
                    page = PageHelper.startPage(1, pageSize).doSelectPageInfo(() -> {
                        medicineStorageDao.listMedicineOutbound(clinic.getClinicId(), startDate, endDate, dateType, count);
                    });
                    System.out.println(clinic.getStoreCode() + "======查询西药销售退回入库列表======total=" + page.getTotal());
                    break;
                case CHI_MED_INBOUND:
                    page = PageHelper.startPage(1, pageSize).doSelectPageInfo(() -> {
                        medicineStorageDao.listChinaMedicineOutbound(clinic.getClinicId(), startDate, endDate, dateType, count);
                    });
                    System.out.println(clinic.getStoreCode() + "======中药销售退回入库列表======total=" + page.getTotal());
                    break;
                case MED_INV:
                    page = PageHelper.startPage(1, pageSize).doSelectPageInfo(() -> {
                        medicineStorageDao.listMedicineInventory(clinic.getClinicId(), startDate, endDate, dateType, count);
                    });
                    System.out.println(clinic.getStoreCode() + "======西药盘盈入库列表======total=" + page.getTotal());
                    break;
                case CHI_MED_INV:
                    page = PageHelper.startPage(1, pageSize).doSelectPageInfo(() -> {
                        medicineStorageDao.listChinaMedicineInventory(clinic.getClinicId(), startDate, endDate, dateType, count);
                    });
                    System.out.println(clinic.getStoreCode() + "======中药盘盈入库列表======total=" + page.getTotal());
                    break;
            }
            if (page.getTotal() == 0) {
                continue;
            }
            Map<String, List<MedicineStorageTemp>> map =  page.getList()
                    .stream()
                    .collect(Collectors.groupingBy(MedicineStorageTemp::getStoreStorageCode));
            for(Map.Entry<String, List<MedicineStorageTemp>> msMap:map.entrySet()){
                MedicineStorageDto medicineStorageDto = new MedicineStorageDto();
                medicineStorageDto.setStoreCode(clinic.getStoreCode());
                switch (flag) {
                    case MED_PUR:
                    case CHI_MED_PUR:
                        medicineStorageDto.setStorePurchaseType("5");
                        medicineStorageDto.setStorageType(StorageType.PUR);
                        break;
                    case MED_MED_INBOUND:
                    case CHI_MED_INBOUND:
                        medicineStorageDto.setStorageType(StorageType.INBOUND);
                        break;
                    case MED_INV:
                    case CHI_MED_INV:
                        medicineStorageDto.setStorageType(StorageType.CHI_MED_INBOUND);
                        break;
                }
                medicineStorageDto.setStorePrecordCode("00000000");
                medicineStorageDto.setStorageMan(msMap.getValue().get(0).getStorageMan());
                medicineStorageDto.setStorageTime(msMap.getValue().get(0).getStorageTime());
                medicineStorageDto.setStoreCode(clinic.getStoreCode());
                medicineStorageDto.setStoreStorageCode(msMap.getKey());
                List<MedicineStorage> list = msMap.getValue()
                        .stream()
                        .map(MedicineStorageTemp::temp2MedSto)
                        .collect(Collectors.toList());

                // 西药部分 耗材西药分别上传 中药不需要处理
                if(flag.equals(MED_PUR) || flag.equals(MED_MED_INBOUND) || flag.equals(MED_INV)){
                    Map<String, List<MedicineStorage>> medMap = list.stream().collect(Collectors.groupingBy(MedicineStorage::getItemType));
                    for(Map.Entry<String, List<MedicineStorage>> m : medMap.entrySet()) {
                        if(m.getValue().size() > 0) {
                            medicineStorageDto.setMedicineInfos(m.getValue());
                            this.upload(medicineStorageDto, flag);
                        }
                    }
                }  else {
                    medicineStorageDto.setMedicineInfos(list);
                    this.upload(medicineStorageDto, flag);
                }
            }
            if (page.getPages() > 1) {
                PageInfo<MedicineStorageTemp> page1 = new PageInfo<>();
                for(int i = 2; i <= page.getPages(); i++) {
                    switch (flag) {
                        case MED_PUR:
                            page1 = PageHelper.startPage(i, pageSize).doSelectPageInfo(() -> {
                                medicineStorageDao.listMedicinePurchase(clinic.getClinicId(), startDate, endDate, dateType, count);
                            });
                            System.out.println(clinic.getStoreCode() + "======查询西药采购入库分页" + i + "======total=" + page.getTotal());
                            break;
                        case CHI_MED_PUR:
                            page1 = PageHelper.startPage(i, pageSize).doSelectPageInfo(() -> {
                                medicineStorageDao.listChinaMedicinePurchase(clinic.getClinicId(), startDate, endDate, dateType, count);
                            });
                            System.out.println(clinic.getStoreCode() + "======查询中药采购入库分页" + i + "======total=" + page.getTotal() );
                            break;
                        case MED_MED_INBOUND:
                            page1 = PageHelper.startPage(i, pageSize).doSelectPageInfo(() -> {
                                medicineStorageDao.listMedicineOutbound(clinic.getClinicId(), startDate, endDate, dateType, count);
                            });
                            System.out.println(clinic.getStoreCode() + "======查询西药销售退回入库分页" + i + "======total=" + page.getTotal());
                            break;
                        case CHI_MED_INBOUND:
                            page1 = PageHelper.startPage(i, pageSize).doSelectPageInfo(() -> {
                                medicineStorageDao.listChinaMedicineOutbound(clinic.getClinicId(), startDate, endDate, dateType, count);
                            });
                            System.out.println(clinic.getStoreCode() + "======查询中药销售退回入库分页" + i + "======total=" + page.getTotal());
                            break;
                        case MED_INV:
                            page1 = PageHelper.startPage(i, pageSize).doSelectPageInfo(() -> {
                                medicineStorageDao.listMedicineInventory(clinic.getClinicId(), startDate, endDate, dateType, count);
                            });
                            System.out.println(clinic.getStoreCode() + "======查询西药盘盈入库分页" + i + "======total=" + page.getTotal());
                            break;
                        case CHI_MED_INV:
                            page1 = PageHelper.startPage(i, pageSize).doSelectPageInfo(() -> {
                                medicineStorageDao.listChinaMedicineInventory(clinic.getClinicId(), startDate, endDate, dateType, count);
                            });
                            System.out.println(clinic.getStoreCode() + "======查询中药销售退回入库上传分页" + i + "======total=" + page.getTotal());
                            break;
                    }
                    Map<String, List<MedicineStorageTemp>> map1 =  page1.getList()
                            .stream()
                            .collect(Collectors.groupingBy(MedicineStorageTemp::getStoreStorageCode));
                    for(Map.Entry<String, List<MedicineStorageTemp>> msMap:map1.entrySet()){
                        MedicineStorageDto medicineStorageDto = new MedicineStorageDto();
                        medicineStorageDto.setStoreCode(clinic.getStoreCode());
                        switch (flag) {
                            case MED_PUR:
                            case CHI_MED_PUR:
                                medicineStorageDto.setStorePurchaseType("5");
                                medicineStorageDto.setStorageType(StorageType.PUR);
                                break;
                            case MED_MED_INBOUND:
                            case CHI_MED_INBOUND:
                                medicineStorageDto.setStorageType(StorageType.INBOUND);
                                break;
                            case MED_INV:
                            case CHI_MED_INV:
                                medicineStorageDto.setStorageType(StorageType.CHI_MED_INBOUND);
                                break;
                        }
                        medicineStorageDto.setStorePrecordCode("00000000");
                        medicineStorageDto.setStorageMan(msMap.getValue().get(0).getStorageMan());
                        medicineStorageDto.setStorageTime(msMap.getValue().get(0).getStorageTime());
                        medicineStorageDto.setStoreCode(clinic.getStoreCode());
                        medicineStorageDto.setStoreStorageCode(msMap.getKey());
                        List<MedicineStorage> list = msMap.getValue()
                                .stream()
                                .map(MedicineStorageTemp::temp2MedSto)
                                .collect(Collectors.toList());
                        if(flag.equals(MED_PUR) || flag.equals(MED_MED_INBOUND) || flag.equals(MED_INV)){
                            Map<String, List<MedicineStorage>> medMap = list.stream().collect(Collectors.groupingBy(MedicineStorage::getItemType));
                            for(Map.Entry<String, List<MedicineStorage>> m : medMap.entrySet()) {
                                System.out.println(medicineStorageDto.getStoreCode() + "======" + ("1".equals(m.getKey())?"药品":"耗材"));
                                if(m.getValue().size() > 0) {
                                    medicineStorageDto.setMedicineInfos(m.getValue());
                                    this.upload(medicineStorageDto, flag);
                                }
                            }
                        }  else {
                            medicineStorageDto.setMedicineInfos(list);
                            this.upload(medicineStorageDto, flag);
                        }
                    }
                }
            }
        };
        long syncTime = System.currentTimeMillis();
        log.info("入库上传用时--{}", startTime - syncTime);
    };

    public void upload (MedicineStorageDto storageDtoInfo, String flag) throws InterruptedException, JsonProcessingException {
        String flagStr = "";
        switch (flag) {
            case MED_PUR:
                flagStr = "西药采购入库";
                break;
            case CHI_MED_PUR:
                flagStr = "中药采购入库";
                break;
            case MED_MED_INBOUND:
                flagStr = "西药销售退回入库";
                break;
            case CHI_MED_INBOUND:
                flagStr = "中药销售退回";
                break;
            case MED_INV:
                flagStr = "西药盘盈入库";
                break;
            case CHI_MED_INV:
                flagStr = "中药盘盈入库";
                break;
        }
        ObjectMapper objectMapper = new ObjectMapper();
        String params = objectMapper.writeValueAsString(storageDtoInfo);
        BaseResult baseResult = RestTemplateUtil.postHttp(url, params);
        if (null != baseResult && baseResult.isSuccess()) {
            System.out.println(flagStr + "======" + storageDtoInfo.getStoreCode() + "======" + baseResult.getMessage());
        } else {
            log.error(flagStr + "------" + storageDtoInfo.getStoreCode() + "------" + baseResult.getMessage());
        }
    }
}
