package com.miyzh.service;

import com.fasterxml.jackson.databind.ObjectMapper;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.miyzh.dao.ClinicDao;
import com.miyzh.dao.MedicineStoreDao;
import com.miyzh.entity.BaseResult;
import com.miyzh.entity.Clinic;
import com.miyzh.entity.MedicineStore;
import com.miyzh.entity.dto.MedicineStoreDto;
import com.miyzh.utils.RestTemplateUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Service;

import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * 库存信息同步
 */

@Slf4j
@Service
public class MedicineStoreService {

    @Autowired
    private ClinicDao clinicDao;
    @Autowired
    private MedicineStoreDao medicineStoreDao;
    @Value ("${postApi.storeUrl}")
    private String url;
    @Value("${postApi.pageSize}")
    private int pageSize;
    @Value("${update.startDate}")
    public String startDate;
    @Value("${update.endDate}")
    public String endDate;
    @Value("${update.dateType}")
    public String dateType;
    @Value("${update.count}")
    public Integer count;

    @Async
    public void uploadMedicine(List<Clinic> clinicList) throws Exception {
        long startTime = System.currentTimeMillis();
        MedicineStoreDto storeInfo = new MedicineStoreDto();
        storeInfo.setCheckMan("");
        storeInfo.setCheckTime(LocalDateTime.now().format(DateTimeFormatter.ofPattern("yyyy-MM-dd hh:mm:ss")));
        for(Clinic clinic : clinicList) {
            System.out.println(clinic.getStoreCode());
            storeInfo.setStoreCode(clinic.getStoreCode());
            PageInfo<MedicineStore> page = PageHelper.startPage(1, pageSize).doSelectPageInfo(() -> {
                medicineStoreDao.listMedicineStore(clinic.getClinicId(), startDate, endDate, dateType, count);
            });
            System.out.println(clinic.getStoreCode() + "======西药库存记录======total=" + page.getTotal());
            if (page.getTotal() == 0) {
                continue;
            }
            Map<String, List<MedicineStore>> map = page.getList().stream().collect(Collectors.groupingBy(MedicineStore::getItemType));
            for(Map.Entry<String, List<MedicineStore>> m : map.entrySet()) {
                if(m.getValue().size() > 0) {
                    System.out.println(storeInfo.getStoreCode() + "======" + ("1".equals(m.getKey())?"药品":"耗材"));
                    storeInfo.setMedicineInfos(m.getValue());
                    this.upload(storeInfo, "西药库存同步");
                }
            }
            if (page.getPages() > 1) {
                for(int i = 2; i <= page.getPages(); i++) {
                    PageHelper.startPage(i, pageSize).doSelectPageInfo(() -> {
                        medicineStoreDao.listMedicineStore(clinic.getClinicId(), startDate, endDate, dateType, count);
                    });
                    System.out.println(clinic.getStoreCode() + "======西药库存记录分页======" + i + "total=" + page.getTotal());
                    Map<String, List<MedicineStore>> map1 = page.getList().stream().collect(Collectors.groupingBy(MedicineStore::getItemType));
                    for(Map.Entry<String, List<MedicineStore>> m : map1.entrySet()) {
                        if(m.getValue().size() > 0) {
                            System.out.println(storeInfo.getStoreCode() + "======" + ("1".equals(m.getKey())?"药品":"耗材"));
                            storeInfo.setMedicineInfos(m.getValue());
                            this.upload(storeInfo, "西药库存分页同步");
                        }
                    }
                }
            }
        };
        long syncTime = System.currentTimeMillis();
        log.info("西药库存同步方法执行用时--{}", startTime - syncTime);
    };

    @Async
    public void uploadChinaMedicine(List<Clinic> clinicList) throws Exception {
        long startTime = System.currentTimeMillis();
        MedicineStoreDto storeInfo = new MedicineStoreDto();
        storeInfo.setCheckMan("");
        storeInfo.setCheckTime(LocalDateTime.now().format(DateTimeFormatter.ofPattern("yyyy-MM-dd hh:mm:ss")));
        for(Clinic clinic : clinicList) {
            System.out.println(clinic.getStoreCode());
            storeInfo.setStoreCode(clinic.getStoreCode());
            PageInfo<MedicineStore> page = PageHelper.startPage(1, pageSize).doSelectPageInfo(() -> {
                medicineStoreDao.listChinaMedicineStore(clinic.getClinicId(), startDate, endDate, dateType, count);
            });
            System.out.println(clinic.getStoreCode() + "======中药库存记录======total=" + page.getTotal());
            if (page.getTotal() == 0) {
                continue;
            }
            storeInfo.setMedicineInfos(page.getList());
            this.upload(storeInfo, "中药库存同步");
            if (page.getPages() > 1) {
                for(int i = 2; i <= page.getPages(); i++) {
                    PageHelper.startPage(i, pageSize).doSelectPageInfo(() -> {
                        medicineStoreDao.listChinaMedicineStore(clinic.getClinicId(), startDate, endDate, dateType, count);
                    });
                    System.out.println(clinic.getStoreCode() + "======中药库存记录分页======" + i + "total=" + page.getTotal());
                    storeInfo.setMedicineInfos(page.getList());
                    this.upload(storeInfo, "中药库存分页同步");
                }
            }
        };
        long syncTime = System.currentTimeMillis();
        log.info("中药库存同步方法执行用时--{}", startTime - syncTime);
    };

    public void upload (MedicineStoreDto storeInfo, String flag) throws Exception {
        ObjectMapper objectMapper = new ObjectMapper();
        String params = objectMapper.writeValueAsString(storeInfo);
        BaseResult baseResult = RestTemplateUtil.postHttp(url, params);
        if (null != baseResult && baseResult.isSuccess()) {
            System.out.println(flag + "======" + storeInfo.getStoreCode() + "======" + baseResult.getMessage());
        } else {
            log.error(flag + "------" + storeInfo.getStoreCode() + "------" + baseResult.getMessage());
        }


    }

}
