package com.miyzh.service;

import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.miyzh.dao.ClinicDao;
import com.miyzh.dao.OrderDetailDao;
import com.miyzh.entity.*;
import com.miyzh.entity.dto.MedicineReturnDto;
import com.miyzh.entity.dto.OrderDetailDto;
import com.miyzh.utils.RestTemplateUtil;
import com.mysql.cj.util.StringUtils;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Service;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * 结算明细
 */

@Slf4j
@Service
public class OrderDetailService {

    @Autowired
    private OrderDetailDao orderDetailDao;
    @Value ("${postApi.orderUrl}")
    private String url;
    @Value("${postApi.pageSize}")
    private int pageSize;
    @Value("${update.startDate}")
    public String startDate;
    @Value("${update.endDate}")
    public String endDate;
    @Value("${update.dateType}")
    public String dateType;
    @Value("${update.count}")
    public Integer count;

    private static final String CLI_MED = "1"; // 西药销售明细(自费门诊)
    private static final String CLI_CHI_MED = "2"; // 中药销售明细(自费门诊)
    private static final String INS_CLC_MED = "3"; // 西药销售明细(医保门诊)
    private static final String INS_CLC_CHI_MED = "4"; // 中药销售明细(医保门诊)
    private static final String HOS_MED = "5"; // 西药销售明细(住院)
    private static final String HOS_CHI_MED = "6"; // 中药销售明细(住院)

    @Async
    public void uploadMedicine(List<Clinic> clinicList, String flag) throws InterruptedException, JsonProcessingException {
        long startTime = System.currentTimeMillis();
        PageInfo<OrderDetailTemp> page = new PageInfo<>();
        for(Clinic clinic : clinicList) {
            switch (flag) {
                case CLI_MED:
                    page = PageHelper.startPage(1, pageSize).doSelectPageInfo(() -> {
                        orderDetailDao.listClcMedOrder(clinic.getClinicId(), startDate, endDate, dateType, count);
                    });
                    System.out.println(clinic.getStoreCode() + "======查询西药销售明细(自费门诊)======total=" + page.getTotal());
                    break;
                case CLI_CHI_MED:
                    page = PageHelper.startPage(1, pageSize).doSelectPageInfo(() -> {
                        orderDetailDao.listClcChinaMedOrder(clinic.getClinicId(), startDate, endDate, dateType, count);
                    });
                    System.out.println(clinic.getStoreCode() + "======查询中药销售明细(自费门诊)======total=" + page.getTotal());
                    break;
                case INS_CLC_MED:
                    page = PageHelper.startPage(1, pageSize).doSelectPageInfo(() -> {
                        orderDetailDao.listInsClcMedOrder(clinic.getClinicId(), startDate, endDate, dateType, count);
                    });
                    System.out.println(clinic.getStoreCode() + "======查询西药销售明细(医保门诊)======total=" + page.getTotal());
                    break;
                case INS_CLC_CHI_MED:
                    page = PageHelper.startPage(1, pageSize).doSelectPageInfo(() -> {
                        orderDetailDao.listInsClcChinaMedOrder(clinic.getClinicId(), startDate, endDate, dateType, count);
                    });
                    System.out.println(clinic.getStoreCode() + "======查询中药销售明细(医保门诊)======total=" + page.getTotal());
                    break;
                case HOS_MED:
                    page = PageHelper.startPage(1, pageSize).doSelectPageInfo(() -> {
                        orderDetailDao.listHosMedOrder(clinic.getClinicId(), startDate, endDate, dateType, count);
                    });
                    System.out.println(clinic.getStoreCode() + "======查询西药销售明细(住院)======total=" + page.getTotal());
                    break;
                case HOS_CHI_MED:
                    page = PageHelper.startPage(1, pageSize).doSelectPageInfo(() -> {
                        orderDetailDao.listHosChinaMedOrder(clinic.getClinicId(), startDate, endDate, dateType, count);
                    });
                    System.out.println(clinic.getStoreCode() + "======查询中药销售明细(住院)======total=" + page.getTotal());
                    break;
            }
            if (page.getTotal() == 0) {
                continue;
            }
            Map<String, List<OrderDetailTemp>> map = new HashMap<>();
//            if (HOS_MED.equals(flag) || HOS_CHI_MED.equals(flag)) {
//                map =  page.getList().stream().collect(Collectors.groupingBy(OrderDetailTemp::getVisitCode));
//            } else {
//                map =  page.getList().stream().collect(Collectors.groupingBy(OrderDetailTemp::getStoreOrderNo));
//            }
            map =  page.getList().stream().collect(Collectors.groupingBy(OrderDetailTemp::getStoreOrderNo));

            for(Map.Entry<String, List<OrderDetailTemp>> msMap:map.entrySet()){
                OrderDetailDto orderDetailDto = new OrderDetailDto();
                orderDetailDto.setStoreCode(clinic.getStoreCode());
                orderDetailDto.setStoreOrderNo(msMap.getValue().get(0).getStoreOrderNo());
                if (StringUtils.isNullOrEmpty(msMap.getValue().get(0).getMisNo())) {
                    orderDetailDto.setMisNo("99999999");
                } else {
                    orderDetailDto.setMisNo(msMap.getValue().get(0).getMisNo());
                }
                orderDetailDto.setSettledTypeNo(msMap.getValue().get(0).getSettledTypeNo());
                orderDetailDto.setSettledTypeName(msMap.getValue().get(0).getSettledTypeName());
                orderDetailDto.setSettledstatus(msMap.getValue().get(0).getSettledstatus());
                orderDetailDto.setTotalAmt(msMap.getValue().get(0).getTotalAmt());
                orderDetailDto.setInsuranceAmt(msMap.getValue().get(0).getInsuranceAmt());
                orderDetailDto.setOperater(msMap.getValue().get(0).getOperater());
                orderDetailDto.setSaleDate(msMap.getValue().get(0).getSaleDate());
                orderDetailDto.setSaleTime(msMap.getValue().get(0).getSaleTime());
                orderDetailDto.setBuyMedPerson(msMap.getValue().get(0).getBuyMedPerson());
                orderDetailDto.setSex(msMap.getValue().get(0).getSex());
                orderDetailDto.setAge(msMap.getValue().get(0).getAge());
                orderDetailDto.setIdCard(msMap.getValue().get(0).getIdCard());
                orderDetailDto.setCredentialNum(msMap.getValue().get(0).getCredentialNum());
                List<OrderDetail> list = msMap.getValue()
                        .stream()
                        .map(OrderDetailTemp::temp2OrderDetail)
                        .collect(Collectors.toList());
                if(flag.equals(CLI_MED) || flag.equals(INS_CLC_MED) || flag.equals(HOS_MED)){
                    Map<String, List<OrderDetail>> medMap = list.stream().collect(Collectors.groupingBy(OrderDetail::getItemType));
                    for(Map.Entry<String, List<OrderDetail>> m : medMap.entrySet()) {
                        if(m.getValue().size() > 0) {
                            orderDetailDto.setMedicineInfos(m.getValue());
                            this.upload(orderDetailDto, flag);
                        }
                    }
                }  else {
                    orderDetailDto.setMedicineInfos(list);
                    this.upload(orderDetailDto, flag);
                }
            }
            if (page.getPages() > 1) {
                PageInfo<OrderDetailTemp> page1 = new PageInfo<>();
                for(int i = 2; i <= page.getPages(); i++) {
                    switch (flag) {
                        case CLI_MED:
                            page1 = PageHelper.startPage(1, pageSize).doSelectPageInfo(() -> {
                                orderDetailDao.listClcMedOrder(clinic.getClinicId(), startDate, endDate, dateType, count);
                            });
                            System.out.println(clinic.getStoreCode() + "======查询西药销售明细(自费门诊)分页" + i + "======total=" + page.getTotal());
                            break;
                        case CLI_CHI_MED:
                            page1 = PageHelper.startPage(1, pageSize).doSelectPageInfo(() -> {
                                orderDetailDao.listClcChinaMedOrder(clinic.getClinicId(), startDate, endDate, dateType, count);
                            });
                            System.out.println(clinic.getStoreCode() + "======查询中药销售明细(自费门诊)分页" + i + "======total=" + page.getTotal());
                            break;
                        case INS_CLC_MED:
                            page1 = PageHelper.startPage(1, pageSize).doSelectPageInfo(() -> {
                                orderDetailDao.listInsClcMedOrder(clinic.getClinicId(), startDate, endDate, dateType, count);
                            });
                            System.out.println(clinic.getStoreCode() + "======查询西药销售明细(医保门诊)分页" + i + "======total=" + page.getTotal());
                            break;
                        case INS_CLC_CHI_MED:
                            page1 = PageHelper.startPage(1, pageSize).doSelectPageInfo(() -> {
                                orderDetailDao.listInsClcChinaMedOrder(clinic.getClinicId(), startDate, endDate, dateType, count);
                            });
                            System.out.println(clinic.getStoreCode() + "======查询中药销售明细(医保门诊)分页" + i + "======total=" + page.getTotal());
                            break;
                        case HOS_MED:
                            page1 = PageHelper.startPage(1, pageSize).doSelectPageInfo(() -> {
                                orderDetailDao.listHosMedOrder(clinic.getClinicId(), startDate, endDate, dateType, count);
                            });
                            System.out.println(clinic.getStoreCode() + "======查询西药销售明细(住院)分页" + i + "======total=" + page.getTotal());
                            break;
                        case HOS_CHI_MED:
                            page1 = PageHelper.startPage(1, pageSize).doSelectPageInfo(() -> {
                                orderDetailDao.listHosChinaMedOrder(clinic.getClinicId(), startDate, endDate, dateType, count);
                            });
                            System.out.println(clinic.getStoreCode() + "======查询中药销售明细(住院)分页" + i + "======total=" + page.getTotal());
                            break;
                    }
                    Map<String, List<OrderDetailTemp>> map1 = new HashMap<>();
                    if (HOS_MED.equals(flag) || HOS_CHI_MED.equals(flag)) {
                        map1 =  page1.getList().stream().collect(Collectors.groupingBy(OrderDetailTemp::getVisitCode));
                    } else {
                        map1 =  page1.getList().stream().collect(Collectors.groupingBy(OrderDetailTemp::getStoreOrderNo));
                    }
                    for(Map.Entry<String, List<OrderDetailTemp>> msMap:map1.entrySet()){
                        OrderDetailDto orderDetailDto = new OrderDetailDto();
                        orderDetailDto.setStoreCode(clinic.getStoreCode());
                        if (HOS_MED.equals(flag) || HOS_CHI_MED.equals(flag)) {
                            orderDetailDto.setStoreOrderNo(msMap.getValue().get(0).getVisitCode());
                        } else {
                            orderDetailDto.setStoreOrderNo(msMap.getValue().get(0).getStoreOrderNo());
                        }
                        if (StringUtils.isNullOrEmpty(msMap.getValue().get(0).getMisNo())) {
                            orderDetailDto.setMisNo("99999999");
                        } else {
                            orderDetailDto.setMisNo(msMap.getValue().get(0).getMisNo());
                        }
                        orderDetailDto.setSettledTypeNo(msMap.getValue().get(0).getSettledTypeNo());
                        orderDetailDto.setSettledTypeName(msMap.getValue().get(0).getSettledTypeName());
                        orderDetailDto.setSettledstatus(msMap.getValue().get(0).getSettledstatus());
                        orderDetailDto.setTotalAmt(msMap.getValue().get(0).getTotalAmt());
                        orderDetailDto.setInsuranceAmt(msMap.getValue().get(0).getInsuranceAmt());
                        orderDetailDto.setOperater(msMap.getValue().get(0).getOperater());
                        orderDetailDto.setSaleDate(msMap.getValue().get(0).getSaleDate());
                        orderDetailDto.setSaleTime(msMap.getValue().get(0).getSaleTime());
                        orderDetailDto.setBuyMedPerson(msMap.getValue().get(0).getBuyMedPerson());
                        orderDetailDto.setSex(msMap.getValue().get(0).getSex());
                        orderDetailDto.setAge(msMap.getValue().get(0).getAge());
                        orderDetailDto.setIdCard(msMap.getValue().get(0).getIdCard());
                        orderDetailDto.setCredentialNum(msMap.getValue().get(0).getCredentialNum());
                        List<OrderDetail> list = msMap.getValue()
                                .stream()
                                .map(OrderDetailTemp::temp2OrderDetail)
                                .collect(Collectors.toList());
                        // 西药部分 耗材西药分别上传 中药不需要处理
                        if(flag.equals(CLI_MED) || flag.equals(INS_CLC_MED) || flag.equals(HOS_MED)){
                            Map<String, List<OrderDetail>> medMap = list.stream().collect(Collectors.groupingBy(OrderDetail::getItemType));
                            for(Map.Entry<String, List<OrderDetail>> m : medMap.entrySet()) {
                                if(m.getValue().size() > 0) {
                                    System.out.println(orderDetailDto.getStoreCode() + "======" + ("1".equals(m.getKey())?"药品":"耗材"));
                                    orderDetailDto.setMedicineInfos(m.getValue());
                                    this.upload(orderDetailDto, flag);
                                }
                            }
                        }  else {
                            orderDetailDto.setMedicineInfos(list);
                            this.upload(orderDetailDto, flag);
                        }
                    }
                }
            }
        };
        long syncTime = System.currentTimeMillis();
        log.info("订单上传执行用时--{}", startTime - syncTime);
    };

    public void upload (OrderDetailDto orderDetailDto, String flag) throws JsonProcessingException {
        String flagStr = "";
        switch (flag) {
            case CLI_MED:
                flagStr = "西药销售明细(自费门诊)";
                break;
            case CLI_CHI_MED:
                flagStr = "中药销售明细(自费门诊)";
                break;
            case INS_CLC_MED:
                flagStr = "西药销售明细(医保门诊)";
                break;
            case INS_CLC_CHI_MED:
                flagStr = "中药销售明细(医保门诊)";
                break;
            case HOS_MED:
                flagStr = "西药销售明细(住院)";
                break;
            case HOS_CHI_MED:
                flagStr = "中药销售明细(住院)";
                break;
        }

        ObjectMapper objectMapper = new ObjectMapper();
        String params = objectMapper.writeValueAsString(orderDetailDto);
        BaseResult baseResult = RestTemplateUtil.postHttp(url, params);
        if (null != baseResult && baseResult.isSuccess()) {
            System.out.println(flagStr + "======" + orderDetailDto.getStoreCode() + "======" + baseResult.getMessage());
        } else {
            log.error(flagStr + "------" + orderDetailDto.getStoreCode() + "------" + baseResult.getMessage());
        }
    }

}
